from caw.commands.store import app

# register subcommands into app
import caw.commands.base
import caw.commands.login
import caw.commands.logout
import caw.commands.upload
import caw.commands.download
import caw.commands.pipeline
import caw.commands.search
