from chris.types import CUBEAddress, CUBEUsername, CUBEPassword

DEFAULT_ADDRESS = CUBEAddress('http://localhost:8000/api/v1/')
DEFAULT_USERNAME = CUBEUsername('chris')
DEFAULT_PASSWORD = CUBEPassword('chris1234')
