from chris.cube.resource.connected_resource import ConnectedResource
from chris.cube.resource.cube_resource import CUBEResource
from chris.cube.resource.templated_resource import ResourceWithTemplate
